package edu.ucf.rp2;

import java.util.HashSet;
import java.util.Iterator;
import java.util.concurrent.CountDownLatch;

public abstract class Tester {
	
	
	protected String logName;
	protected CSVWriter logger;
	protected HashSet<String> encodedCharList;
	protected int maxSampleSize;
	protected int linkID = 1;
	
	
	public void start (String[] searchTerms) {
		logger = new CSVWriter(logName);	
		makeHeader();
		
		while (linkID<maxSampleSize)
		{
			processSearch(searchTerms[linkID-1]);
			//System.out.println(printEncodedChars());
		}
		
		logger.close();	
	}
	
	
	public String[] getEncodedChars () {
		return encodedCharList.toArray(new String[encodedCharList.size()]);
	}
		
	
	protected void compareWebsites (String url1, String url2) {
		double diff = -1;
		
		CountDownLatch cdl = new CountDownLatch(2);
		WebFetcher wf1 = new WebFetcher(url1, cdl);
		WebFetcher wf2 = new WebFetcher(url2, cdl);
		wf1.start();
		wf2.start();
		
		try {
			cdl.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		System.out.println("Processing link: " + linkID);
		System.out.println("\t" + wf1.getStatusCode() + ": " + url1);
		System.out.println("\t" + wf2.getStatusCode() + ": " + url2);
		
		logger.nextField(Integer.toString(wf1.getStatusCode()));
		logger.nextField(Integer.toString(wf2.getStatusCode()));
		if (wf1.isOKCode() && wf2.isOKCode()) {
			diff = DiffLines.perecentDiff(wf1.getSource(), wf2.getSource());
		}
		logger.nextField(Double.toString(diff));
	}
	
	
	protected void processEncodedChars (String[] encodedChars) {
		if (encodedCharList==null) {
			encodedCharList = new HashSet<String>();
		}
		
		for (String encodedChar : encodedChars)
		{
			encodedCharList.add(encodedChar);
		}
	}
	
	
	protected String printEncodedChars () {
		String encodedChars = "<-- ";
		Iterator<String> iter = encodedCharList.iterator();
		while (iter.hasNext())
		{
			encodedChars = encodedChars + iter.next() + " ";
		}
		return encodedChars + "-->";
	}
	
	
	protected abstract void processSearch (String searchTerm);
	//protected abstract void processAd (Ad ad);
	protected abstract void makeHeader();
}
